library EAPS3645;

// Delphi 5 DLL-source
// for ARRAY EA-PS Powersupplies

uses
  SysUtils,
  Windows,
  Classes,
  PSServerProj1_TLB,
  Controls,
  Forms,
  ShellAPI,
  Dlg_EAPS3645 in 'Dlg_EAPS3645.pas' {EAPS_Dlg};

{$R *.RES}

Const  Inputs =  4;  // number of inputs
       Outputs = 3; // number of outputs

       {INPUTS}
       SetVolt = 0; // index of input VOLT
       SetCur  = 1;  //index of input mA
       SetPow  = 2;  //index of input W
       SetOE   = 3;  //index of input OE

       {OUTPUTS}
       GetVolt = 0; // index of output VOLT
       GetCur  = 1;  // index of output mA
       GetPow  = 2;  // index of output W

       {USER}
       ComNo = 0; // index user variable ComNo
       Baud  = 1; // index user variable Baud
       DevAdr= 2; // index user variable DevAdr
       SetVoltAlt= 3; // index user variableSetVoltAlt


Type TDLLParams = array[0..100] of extended;   //Type of ProfiLab DLL parameters
     PDLLParams = ^TDLLParams;                 // Pointer to ProfiLab DLL parameters

var TheSupply: TPSServer;

function NumInputs: Byte;
begin
  result:=Inputs; //Define number of component input pins
end;

function NumOutputs: Byte;
begin
  result:=Outputs; //Define number of component output pins
end;

Function InputName(Channel: Byte): ShortString; // Return name for each component input pin
begin
   case Channel of
   SetVolt: result:='V';   // Volt
   SetCur : result:='A';  // Current
   SetPow : result:='W';   // Watt
   SetOE  : result:='OE';   // Output ON/OFF
   end;
end;

Function OutputName(Channel: Byte): ShortString; // Return name for each component output pin
begin
   case Channel of
   GetVolt: result:='V';   //  Volt
   GetCur : result:='A';  //  Current
   GetPow : result:='W';   //  Watt
   end;
end;

Procedure SimStart(PInput,POutput,PUser: PDLLParams); //called when ProfiLab enters RUN mode
begin
   if not FileExists(extractFilepath(application.exename)+'PSActiveXServer.ocx') then
   begin
      messagebox(0,'Missing file: PSActiveXServer.ocx','Array Power Supply', mb_OK);
      TheSupply:=nil;
      exit;
   end;

   shellExecute(0,nil,'regsvr32',Pchar(extractFilepath(application.exename)+'PSActiveXServer.ocx /s'),nil,sw_hide);
   sleep(500);

   TheSupply:=TPSServer.Create(Application.MainForm);
//   if TheSupply=nil then exit;
   EAPS_Dlg:=TEAPS_Dlg.Create(nil);
   with TheSupply do
   begin
     EAPS_Dlg.Supply:=TheSupply;
     PS_OpenComm(round(PUser^[ComNo]),round(PUser^[Baud]),8,0,0);
     OnReceive:=EAPS_Dlg.Receive;

     //Setting Device Address
     PS_SetDeviceAdd(round(PUser^[DevAdr]));
     sleep(100);

     PS_ControlState(3); //PC Control On
     sleep(500);

     //Get Execute Result
     if PS_GetExecResult >0 then
       POutput^[GetVolt]:=5
     else
       POutput^[GetVolt]:=0;
     sleep(100);
   end;
end;

Procedure SimStop(PInput,POutput,PUser: PDLLParams); //called when ProfiLab RUN mode is terminated
begin
  if TheSupply=nil then exit;
  TheSupply.PS_ControlState(1); //PC Control Off
  Sleep(1000);
  TheSupply.PS_CloseComm;
  EAPS_Dlg.free;
  shellExecute(0,nil,'regsvr32',Pchar(extractFilepath(application.exename)+'PSActiveXServer.ocx /s'),nil,sw_hide);
  TheSupply:=nil;
end;

Procedure Calculate(PInput,POutput,PUser: PDLLParams); //called regularly from ProfiLab
var Cur,Vol,VolMax,Pow, add:longint;
begin
   if TheSupply=nil then exit;
   if getTickCount mod 1000 = 0 then
   begin
      Add:=round(PUser^[DevAdr]);

      Vol:=round(PInput^[SetVolt]*1000);
      Cur:=round(PInput^[SetCur]*1000);
      Pow:=round(PInput^[SetPow]*100);
      VolMax:=36*1000;

      TheSupply.PS_SetParams(Cur,VolMax,Pow,Vol,Add);
      if PInput^[SetOE]>2.5 then TheSupply.PS_ControlState(3) else TheSupply.PS_ControlState(2);
   end
   else
   begin
      if getTickCount mod 500 = 0 then
      begin
         TheSupply.PS_ReadParams;

         POutput^[GetVolt]:=EAPS_Dlg.VolValue/1000;
         POutput^[GetCur]:=EAPS_Dlg.CurValue/1000;
         POutput^[GetPow]:=EAPS_Dlg.PowValue/100;
      end;
   end;
end;


Procedure Configure(PUser: PDLLParams);
begin
   EAPS_Dlg:=TEAPS_Dlg.Create(nil);
   with EAPS_Dlg do
   begin
      cbCom.ItemIndex:=round(PUser^[ComNo]);
      cbBaud.Text:=intToStr(round(PUser^[Baud]));
      spAdr.Value:=round(PUser^[DevAdr]);
      if showModal=mrOK then
      begin
         PUser^[ComNo]:=cbCom.ItemIndex;
         PUser^[Baud]:=StrToInt(cbBaud.Text);
         PUser^[DevAdr]:=spAdr.Value;
      end;
      free;
   end;
end;

//export methods for ProfiLab
exports SimStart,
        SimStop,
        NumInputs,
        NumOutputs,
        Calculate,
        InputName,
        OutputName,
        Configure;

begin
end.

